/* bitmap.h -- bitmap implementation using C macros
 *
 * COPYRIGHT AND PERMISSION NOTICE
 * 
 * Copyright (c) 2001 Salvatore Sanfilippo
 * 
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, provided that the above
 * copyright notice(s) and this permission notice appear in all copies of
 * the Software and that both the above copyright notice(s) and this
 * permission notice appear in supporting documentation.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
 * OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL
 * INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * Except as contained in this notice, the name of a copyright holder
 * shall not be used in advertising or otherwise to promote the sale, use
 * or other dealings in this Software without prior written authorization
 * of the copyright holder.
 */

#ifndef _NB_BITMAP_H
#define _NB_BITMAP_H

#include <string.h>

/* Bitmap definition and macros to set/unset/test bits */
#define NB_BITMAP_BASETYPE unsigned long

#define NB_BITMAP_DECLARE(name,size) \
	NB_BITMAP_BASETYPE name[(size)/sizeof(NB_BITMAP_BASETYPE)]

/* Set the bit of number 'nr' */
#define NB_BITMAP_SET(name, nr) name[(nr)/sizeof(NB_BITMAP_BASETYPE)] |= \
	(1UL << ((nr)%sizeof(NB_BITMAP_BASETYPE)))

/* Unset the bit of number 'nr' */
#define NB_BITMAP_UNSET(name, nr) name[(nr)/sizeof(NB_BITMAP_BASETYPE)] &= \
	~(1UL << ((nr)%sizeof(NB_BITMAP_BASETYPE)))

/* Return non-zero if the bit of number 'nr' is set */
#define NB_BITMAP_TEST(name, nr) ((name[(nr)/sizeof(NB_BITMAP_BASETYPE)]) & \
	(1UL << ((nr)%sizeof(NB_BITMAP_BASETYPE))))

/* Initialize the bitmap to 0 */
#define NB_BITMAP_INIT(name, c) memset(name, c, sizeof(name))

#endif
